==> botanypots-fabric-1.21.1-21.1.1-sources.jar: net/darkhax/botanypots/common/impl/data/recipe/soil/BasicSoil.java <==
package net.darkhax.botanypots.common.impl.data.recipe.soil;

import com.mojang.serialization.Codec;
import com.mojang.serialization.MapCodec;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import net.darkhax.bookshelf.common.api.data.codecs.stream.StreamCodecs;
import net.darkhax.bookshelf.common.api.util.DataHelper;
import net.darkhax.botanypots.common.api.data.context.BotanyPotContext;
import net.darkhax.botanypots.common.api.data.display.types.Display;
import net.darkhax.botanypots.common.api.data.display.types.DisplayType;
import net.darkhax.botanypots.common.api.data.recipes.CacheableRecipe;
import net.darkhax.botanypots.common.api.data.recipes.soil.Soil;
import net.darkhax.botanypots.common.impl.Helpers;
import net.minecraft.class_1799;
import net.minecraft.class_1856;
import net.minecraft.class_1865;
import net.minecraft.class_1937;
import net.minecraft.class_2561;
import net.minecraft.class_9129;
import net.minecraft.class_9135;
import net.minecraft.class_9139;
import org.jetbrains.annotations.NotNull;

import java.util.function.Consumer;

public class BasicSoil extends Soil implements CacheableRecipe {

    public static final MapCodec<BasicSoil> CODEC = Properties.CODEC.xmap(BasicSoil::new, BasicSoil::getProperties);
    public static final class_9139<class_9129, BasicSoil> STREAM = Properties.STREAM.method_56432(BasicSoil::new, BasicSoil::getProperties);
    public static final class_1865<BasicSoil> SERIALIZER = DataHelper.recipeSerializer(CODEC, STREAM);

    private final Properties properties;

    public BasicSoil(Properties properties) {
        this.properties = properties;
    }

    public Properties getProperties() {
        return this.properties;
    }

    @Override
    public boolean matches(@NotNull BotanyPotContext input, @NotNull class_1937 level) {
        return this.properties.input.method_8093(input.getSoilItem());
    }

    @Override
    public boolean canBeCached() {
        return true;
    }

    @Override
    public boolean isCacheKey(class_1799 stack) {
        return this.properties.input.method_8093(stack);
    }

    @Override
    public float getGrowthModifier(@NotNull BotanyPotContext context, @NotNull class_1937 level) {
        return this.properties.growthModifier;
    }

    @Override
    public int getLightLevel(@NotNull BotanyPotContext context, @NotNull class_1937 level) {
        return this.properties.lightLevel;
    }

    @Override
    public Display getDisplay(BotanyPotContext context, @NotNull class_1937 level) {
        return this.properties.display;
    }

    @NotNull
    @Override
    public class_1865<?> method_8119() {
        return SERIALIZER;
    }

    @Override
    public void hoverTooltip(class_1799 stack, BotanyPotContext context, class_1937 level, Consumer<class_2561> tooltipLines) {
        if (this.properties.growthModifier != 0f) {
            tooltipLines.accept(Helpers.growthModifierComponent(this.properties.growthModifier));
        }
    }

    @Override
    public boolean couldMatch(class_1799 candidate, BotanyPotContext context, class_1937 level) {
        return this.properties.input.method_8093(candidate);
    }

    public record Properties(class_1856 input, Display display, float growthModifier, int lightLevel) {
        public static final MapCodec<Properties> CODEC = RecordCodecBuilder.mapCodec(instance -> instance.group(
                class_1856.field_46095.fieldOf("input").forGetter(Properties::input),
                DisplayType.DISPLAY_STATE_CODEC.fieldOf("display").forGetter(Properties::display),
                Codec.FLOAT.optionalFieldOf("growth_modifier", 0f).forGetter(Properties::growthModifier),
                Codec.INT.optionalFieldOf("light_level", 0).forGetter(Properties::lightLevel)
        ).apply(instance, Properties::new));
        public static final class_9139<class_9129, Properties> STREAM = class_9139.method_56905(
                StreamCodecs.INGREDIENT_NON_EMPTY, Properties::input,
                DisplayType.DISPLAY_STATE_STREAM, Properties::display,
                class_9135.field_48552, Properties::growthModifier,
                class_9135.field_49675, Properties::lightLevel,
                Properties::new
        );
    }
}

==> botanypots-fabric-1.21.1-21.1.16-sources.jar: net/darkhax/botanypots/common/impl/data/recipe/soil/BasicSoil.java <==
package net.darkhax.botanypots.common.impl.data.recipe.soil;

import com.mojang.serialization.Codec;
import com.mojang.serialization.MapCodec;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import net.darkhax.bookshelf.common.api.data.codecs.stream.StreamCodecs;
import net.darkhax.bookshelf.common.api.util.DataHelper;
import net.darkhax.botanypots.common.api.context.BotanyPotContext;
import net.darkhax.botanypots.common.api.data.display.types.Display;
import net.darkhax.botanypots.common.api.data.display.types.DisplayType;
import net.darkhax.botanypots.common.api.data.recipes.CacheableRecipe;
import net.darkhax.botanypots.common.api.data.recipes.soil.Soil;
import net.darkhax.botanypots.common.impl.Helpers;
import net.minecraft.class_1799;
import net.minecraft.class_1856;
import net.minecraft.class_1865;
import net.minecraft.class_1937;
import net.minecraft.class_2561;
import net.minecraft.class_9129;
import net.minecraft.class_9135;
import net.minecraft.class_9139;
import org.jetbrains.annotations.NotNull;

import java.util.function.Consumer;

public class BasicSoil extends Soil implements CacheableRecipe {

    public static final MapCodec<BasicSoil> CODEC = Properties.CODEC.xmap(BasicSoil::new, BasicSoil::getProperties);
    public static final class_9139<class_9129, BasicSoil> STREAM = Properties.STREAM.method_56432(BasicSoil::new, BasicSoil::getProperties);
    public static final class_1865<BasicSoil> SERIALIZER = DataHelper.recipeSerializer(CODEC, STREAM);

    private final Properties properties;

    public BasicSoil(Properties properties) {
        this.properties = properties;
    }

    public Properties getProperties() {
        return this.properties;
    }

    @Override
    public boolean matches(@NotNull BotanyPotContext input, @NotNull class_1937 level) {
        return this.properties.input.method_8093(input.getSoilItem());
    }

    @Override
    public boolean canBeCached() {
        return true;
    }

    @Override
    public boolean isCacheKey(class_1799 stack) {
        return this.properties.input.method_8093(stack);
    }

    @Override
    public float getGrowthModifier(@NotNull BotanyPotContext context, @NotNull class_1937 level) {
        return this.properties.growthModifier;
    }

    @Override
    public int getLightLevel(@NotNull BotanyPotContext context, @NotNull class_1937 level) {
        return this.properties.lightLevel;
    }

    @Override
    public Display getDisplay(BotanyPotContext context, @NotNull class_1937 level) {
        return this.properties.display;
    }

    @NotNull
    @Override
    public class_1865<?> method_8119() {
        return SERIALIZER;
    }

    @Override
    public void hoverTooltip(class_1799 stack, BotanyPotContext context, class_1937 level, Consumer<class_2561> tooltipLines) {
        if (this.properties.growthModifier != 0f) {
            tooltipLines.accept(Helpers.growthModifierComponent(this.properties.growthModifier));
        }
    }

    @Override
    public boolean couldMatch(class_1799 candidate, BotanyPotContext context, class_1937 level) {
        return this.properties.input.method_8093(candidate);
    }

    public record Properties(class_1856 input, Display display, float growthModifier, int lightLevel) {
        public static final MapCodec<Properties> CODEC = RecordCodecBuilder.mapCodec(instance -> instance.group(
                class_1856.field_46095.fieldOf("input").forGetter(Properties::input),
                DisplayType.DISPLAY_STATE_CODEC.fieldOf("display").forGetter(Properties::display),
                Codec.FLOAT.optionalFieldOf("growth_modifier", 0f).forGetter(Properties::growthModifier),
                Codec.INT.optionalFieldOf("light_level", 0).forGetter(Properties::lightLevel)
        ).apply(instance, Properties::new));
        public static final class_9139<class_9129, Properties> STREAM = class_9139.method_56905(
                StreamCodecs.INGREDIENT_NON_EMPTY, Properties::input,
                DisplayType.DISPLAY_STATE_STREAM, Properties::display,
                class_9135.field_48552, Properties::growthModifier,
                class_9135.field_49675, Properties::lightLevel,
                Properties::new
        );
    }
}
